#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include "uart.h"
#include "global.h"
#include "avrprog.h"

#define XTAL_CPU       11059200 
#define UART_BAUD_RATE    19200 

#define AVR_OK		1

void command_mode(void);
unsigned char flags=0, speed=5;
unsigned int size=0,pagesize=0;

int main(void)
{ 
	unsigned int blink,c ;
	unsigned char state=0;
    uart_init(UART_BAUD_SELECT(UART_BAUD_RATE, XTAL_CPU)); 

	sei(); 

	DDRD=254;
	PORTD=1;
	sbi (PORTD, LED);

	init_resetAVR();

	if (read_sigAVR(0)==30)
		{cbi (PORTD, LED);
		flags|=AVR_OK;}

	DDRB=0;

   for(;;)							//Befehlsequenz: 50, 100, 150, 200 zum Eintritt in den Command Mode
	{c= uart_getc();
	if (c&UART_NO_DATA);
	else if ((state==0)&&(c==50))
		state=1;
	else if ((state==1)&&(c==100))
		state=2;
	else if ((state==2)&&(c==150))
		state=3;
	else if ((state==3)&&(c==200))
		{command_mode();
		state=0;}
	else
		state=0;
	
	if (!(flags&AVR_OK))
		{blink++;
		if (blink==0)
			PORTD=PORTD^(1<<LED);}
	}
}

void command_mode(void)				//Hier luft alles ab, wenn Befehle bertragen werden
{
	unsigned char c, special=0;

	sbi (PORTD, LED);

	c = uart_get();

	if (c==10)						//Befehl 10 Read Signature
		{uart_putc(255);
		uart_putc(255);
		uart_putc(255);
		}
	else if (c==11)					//Befehl 11 Erase Device
		{uart_putc(33);
		}
	else if (c==12)					//Befehl 12 Read Flash
		{uart_putc(33);
		}
	else if (c==13)					//Befehl 13 Program Flash
		{uart_putc(33);
		}
	else if (c==14)					//Befehl 14 Init SPI
		{uart_putc(33);
		}
	else if (c==15)					//Befehl 15 Program Lock Bits
		{uart_putc(33);
		}
	else if (c==18)					//Befehl 18 Read Lock Bits
		{uart_putc(255);
		}
	else if (c==19)					//Befehl 19 Start Program
		{cbi(PORTD, LED);
		DDRB=0;
		PORTB=0;
		uart_putc(33);
		}
	else if (c==30)					//Befehl 30	Set Numer of Bytes Low
		{c=uart_get();
		size=(size&65280)|c;
		uart_putc(c);
		}
	else if (c==31)					//Befehl 31	Set Numer of Bytes High
		{c=uart_get();
		size=(size&255)|(256*c);
		uart_putc(c);
		}
	else if (c==32)					//Befehl 32	Set Page Size
		{c=uart_get();
		pagesize=c;
		uart_putc(c);
		}	
	else if (c==40)					//Befehl 40	Read Signature AVR
		{uart_putc(read_sigAVR(0));
		uart_putc(read_sigAVR(1));
		uart_putc(read_sigAVR(2));
		}
	else if (c==41)					//Befehl 41	Erase Device AVR
		{eraseAVR();
		uart_putc(33);
		}
	else if (c==42)					//Befehl 42 Read Flash AVR
		{read_flashAVR(size);	
		uart_putc(33);
		}
	else if (c==43)					//Befehl 43 Program Flash AVR
		{prog_flashAVR(size);
		uart_putc(33);
		}
	else if (c==44)					//Befehl 44 Init SPI AVR
		{sbi(PORTD, LED);
		init_resetAVR();
		uart_putc(33);
		}
	else if (c==45)					//Befehl 45 Program Lock Bits AVR
		{prog_lockbitsAVR(uart_get());
		uart_putc(33);
		}
	else if (c==46)					//Befehl 46 Program Fuse Bits AVR
		{prog_fusebitsAVR(uart_get());
		uart_putc(33);
		}
	else if (c==47)					//Befehl 47 Read Fuse Bits AVR
		{uart_putc(read_fusebitsAVR());
		}
	else if (c==48)					//Befehl 48 Read Lock Bits AVR
		{uart_putc(read_lockbitsAVR());
		}
	else if (c==49)					//Befehl 49 Start Program AVR
		{cbi(PORTD, LED);
		DDRB=0;
		PORTB=0;
		uart_putc(33);
		}
	else if (c==50)					//Befehl 50 Program Page Flash AVR
		{prog_flash_pageAVR(size, pagesize);
		uart_putc(33);
		}
	else if (c==52)					//Befehl 52 Read EEPROM AVR
		{read_eepromAVR(size);
		uart_putc(33);
		}
	else if (c==53)					//Befehl 53 Program EEPROM AVR
		{prog_eepromAVR(size);
		uart_putc(33);
		}
	else if (c==54)					//Befehl 54 Read OscCal AVR
		{uart_putc(read_calAVR(0));
		uart_putc(read_calAVR(1));
		uart_putc(read_calAVR(2));
		uart_putc(read_calAVR(3));
		}
	else if (c==56)					//Befehl 56 Program Fuse Bits High AVR
		{prog_fusebits_hAVR(uart_get());
		uart_putc(33);
		}
	else if (c==57)					//Befehl 57 Read Fuse Bits High AVR
		{uart_putc(read_fusebits_hAVR());
		}
	else if (c==60)					//Befehl 60	Enter Specal Mode 1
		{special=1;
		}
	else if (c==69)					//Befehl 69	Enter Specal Mode 3
		{if (special==2)
			{uart_putc(33);
			special_modeAVR();}
		else
			special=0;
		}
	else if (c==71)					//Befehl 71	Enter Specal Mode 2
		{if (special==1)
			special=2;
		else
			special=0;
		}
}

void delay (unsigned int us)		//Fr 11MHz kalibriert !
{   unsigned int i;
    for (i=0;i<us;i++)
    	{asm ("nop");
		asm ("nop");
		asm ("nop");
		asm ("nop");}
}

unsigned char uart_get(void)
{unsigned int c;
	while ((c= uart_getc())==UART_NO_DATA);
	return (char)(c&255);
}
